/*
 *  
 *  Laminator Version 1.0 - main.c
 *  2005 Sven Guenther
 * 
 *  Licence: BSD
 *
 */

//#include <stdio.h>
// Systemweite Definitionen und Variablen/Konstanten
#include "system.h"
#include <avr/eeprom.h>
#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/pgmspace.h>
#include "uart.h"

/*
 * Defines
 */
#define ENABLE_RELAIS   PORTB |= 0x01;
#define DISABLE_RELAIS  PORTB &= ~0x01;

#define ENABLE_LED1     PORTD |= 0x04;
#define DISABLE_LED1    PORTD &= ~0x04;

#define ENABLE_LED2     PORTD |= 0x08;
#define DISABLE_LED2    PORTD &= ~0x08;


#define TIMER_1_CNT     0xf0be
#define TMC16_CK1024	(_BV(CS12)+_BV(CS10))

#define ON  1
#define OFF 0

/*
 *  Variables
 */ 
char flip=0;
char counter=0;

char oncount=2;
char offcount=15;
char automatic=1;

SIGNAL(SIG_OVERFLOW1) {          // signal handler for tcnt1 overflow interrupt
	if(PINC & (1 << PC0)) {
		uart_puts_P("Manual override!\r\n");
	    ENABLE_RELAIS;
	} else {	
        if(counter<oncount) {
           if(flip==0) {
                ENABLE_RELAIS;
//    			uart_puts_P("EN RELAIS\r\n");
		        flip=1;
            } else {
	    	    DISABLE_RELAIS;
//    		    uart_puts_P("DIS RELAIS\r\n");
    		    flip=0;
	        }
	    }
	    counter++;
	    if(counter==offcount) counter=0;
	}	
    TCNT1 = TIMER_1_CNT;         // reset counter to get this interrupt again
}

/* define CPU frequency in Mhz here if not defined in Makefile */
#ifndef F_CPU
#define F_CPU 8000000UL
#endif

/* 9600 baud */
#define UART_BAUD_RATE      9600      

void prompt(void) {
    uart_putc('\n');
    uart_putc('\r');
    uart_putc('>');
    uart_putc(' ');
}

int main(void)
{
    unsigned int c;
    char buffer[7];

    
    /*
     *  Initialize UART library, pass baudrate and AVR cpu clock
     *  with the macro 
     *  UART_BAUD_SELECT() (normal speed mode )
     *  or 
     *  UART_BAUD_SELECT_DOUBLE_SPEED() ( double speed mode)
     */
    uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU) ); 
    
    /*
     * now enable interrupt, since UART library is interrupt controlled
     */
    sei();
    
    /*
     * Transmit string from program memory to UART
     */
    uart_puts_P("LaminatorControll\n");
    
    TCCR1A = 0x00;               // disable PWM and Compare Output Mode
    TCCR1B = TMC16_CK1024;       // use CLK/1024 prescale value
    TCNT1  = TIMER_1_CNT;        // reset TCNT1
    
    TIMSK  = (1 << TOIE1);         // enable TCNT1 overflow 
    
	DDRB = 0x01; // Relais pin als Ausgabe definieren.
	DDRD = 0x0C; // LEDs pins als Ausgabe definieren.
	DDRC = 0x00; // Port C als eingang Definieren.
	PORTC = 0x0F; 
	
	ENABLE_RELAIS;       
    
	DISABLE_LED1;
	DISABLE_LED2;
	
	DISABLE_RELAIS;
				
	uart_putc( 0x0c );
	
	/*
	 *  Read Default Values from EEprom
	 */
	 
	eeprom_write_byte(0,oncount);
	eeprom_write_byte(1,offcount);
		 
	oncount  = eeprom_read_byte(0); // On Count
	offcount = eeprom_read_byte(1); // OFF Count
	
    /*
     * Transmit single character to UART
     */
    uart_putc('\r');
    uart_putc('>');
    uart_putc(' ');
    
    for(;;) {

        /*
         * Get received character from ringbuffer
         * uart_getc() returns in the lower byte the received character and 
         * in the higher byte (bitmask) the last receive error
         * UART_NO_DATA is returned when no data is available.
         *
         */
        c = uart_getc();
        if ( c & UART_NO_DATA )
        {
            /* 
             * no data available from UART 
             */
        }
        else
        {
            /*
             * new data available from UART
             * check for Frame or Overrun error
             */
            if ( c & UART_FRAME_ERROR )
            {
                /* Framing Error detected, i.e no stop bit detected */
                uart_puts_P("UART Frame Error: ");
            }
            if ( c & UART_OVERRUN_ERROR )
            {
                /* 
                 * Overrun, a character already present in the UART UDR register was 
                 * not read by the interrupt handler before the next character arrived,
                 * one or more received characters have been dropped
                 */
                uart_puts_P("UART Overrun Error: ");
            }
            if ( c & UART_BUFFER_OVERFLOW )
            {
                /* 
                 * We are not reading the receive buffer fast enough,
                 * one or more received character have been dropped 
                 */
                uart_puts_P("Buffer overflow error: ");
            }
            /* 
             * send received character back
             */
            uart_putc( (unsigned char)c );
			
			switch ((unsigned char)c) {
		        
				case 'a' : 
					 automatic=OFF;
					 TIMSK  &= ~(1 << TOIE1);
					 uart_puts_P("\n\rAutomatic mode off.");
					 prompt();
					 break;
				case 'A' :
					 automatic=ON;
					 TIMSK  = (1 << TOIE1);
				 	 uart_puts_P("\n\rAutomatic mode on.");
					 ENABLE_RELAIS;
					 prompt();
				   	 break;

				case 'l' : 
					 ENABLE_LED1;
					 prompt();
					 break;
				case 'L' :
					 DISABLE_LED1;
					 prompt();
				   	 break;
						
				case 'k' : 
					 ENABLE_LED2;
					 prompt();
					 break;
				case 'K' :
					 DISABLE_LED2;
					 prompt();
				   	 break;
						
				case 'r' :
					 ENABLE_RELAIS;
					 prompt();
					 break;
					 
				case 'R' :
					 DISABLE_RELAIS;
					 prompt();
					 break;
				default :
					 prompt();
			}
        }
    }
	
}


